@@ -726,8 +726,12 @@ totem_pl_parser_get_feed_uri (char *data, gsize len, gboolean debug)
 	/* Redirect plist? Find a goto action */
 	uri = totem_pl_parser_parse_plist (doc);
 
-	if (debug)
-		g_print ("Found redirect URL: %s\n", uri);
+	if (debug) {
+		if (uri == NULL)
+			g_print ("Did not find redirect URL in: %.*s\n", (int) len, data);
+		else
+			g_print ("Found redirect URL: %s\n", uri);
+	}
 
 	if (uri == NULL)
 		goto out;
@@ -770,6 +774,10 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 
 	/* Load the file using iTunes user-agent */
 	content = totem_pl_parser_load_http_itunes (itms_uri, totem_pl_parser_is_debugging_enabled (parser));
+	g_free (itms_uri);
+
+	if (!content)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	/* And look in the file for the feedURL */
 	feed_file = totem_pl_parser_get_feed_uri ((char *) content->data, content->len,
