@@ -28,21 +28,63 @@
 #include "totem-pl-parser-videosite.h"
 #include "totem-pl-parser-private.h"
 
-#define BASE 20
+/* The helper script will be either the one shipped in totem-pl-parser,
+ * when running tests, or the first non-hidden file in the totem-pl-parser
+ * libexec directory, when sorted by lexicographic ordering (through strcmp) */
+static char *
+find_helper_script (void)
+{
+#ifdef UNINSTALLED_TESTS
+	return g_strdup ("../99-totem-pl-parser-videosite");
+#else
+	GDir *dir;
+	const char *name;
+	char *script_name = NULL;
+
+	dir = g_dir_open (LIBEXECDIR "/totem-pl-parser", 0, NULL);
+	if (!dir)
+		goto bail;
+
+	while ((name = g_dir_read_name (dir)) != NULL) {
+		/* Skip hidden files */
+		if (name[0] == '.')
+			continue;
+		if (script_name == NULL || g_strcmp0 (name, script_name) < 0) {
+			g_free (script_name);
+			script_name = g_strdup (name);
+		}
+	}
+	g_clear_pointer (&dir, g_dir_close);
+
+	if (script_name != NULL) {
+		char *ret;
+		ret = g_build_filename (LIBEXECDIR "/totem-pl-parser", script_name, NULL);
+		g_free (script_name);
+		return ret;
+	}
+
+bail:
+	return g_strdup (LIBEXECDIR "/totem-pl-parser/99-totem-pl-parser-videosite");
+#endif
+}
 
 gboolean
 totem_pl_parser_is_videosite (const char *uri, gboolean debug)
 {
 #ifdef HAVE_QUVI
 	const char *args[] = {
-		LIBEXECDIR "/totem-pl-parser-videosite",
+		NULL,
 		"--check",
 		"--url",
 		NULL,
 		NULL
 	};
 	char *out;
+	char *script;
 
+	script = find_helper_script ();
+
+	args[0] = script;
 	args[3] = uri;
 	g_spawn_sync (NULL,
 		      (char **) args,
@@ -55,8 +97,10 @@ totem_pl_parser_is_videosite (const char *uri, gboolean debug)
 		      NULL,
 		      NULL);
 	if (debug)
-		g_print ("Checking videosite for URI '%s' returned '%s' (%s)\n",
-			 uri, out, g_strcmp0 (out, "TRUE") == 0 ? "true" : "false");
+		g_print ("Checking videosite with script '%s' for URI '%s' returned '%s' (%s)\n",
+			 script, uri, out, g_strcmp0 (out, "TRUE") == 0 ? "true" : "false");
+
+	g_free (script);
 
 	return (g_strcmp0 (out, "TRUE") == 0);
 #else
@@ -75,7 +119,7 @@ totem_pl_parser_add_videosite (TotemPlParser *parser,
 {
 #ifdef HAVE_QUVI
 	const char *args[] = {
-		LIBEXECDIR "/totem-pl-parser-videosite",
+		NULL,
 		"--url",
 		NULL,
 		NULL
@@ -86,9 +130,13 @@ totem_pl_parser_add_videosite (TotemPlParser *parser,
 	guint i;
 	GHashTable *ht;
 	char *new_uri = NULL;
+	char *script;
+	TotemPlParserResult ret;
 
 	uri = g_file_get_uri (file);
+	script = find_helper_script ();
 
+	args[0] = script;
 	args[2] = uri;
 	g_spawn_sync (NULL,
 		      (char **) args,
@@ -104,13 +152,18 @@ totem_pl_parser_add_videosite (TotemPlParser *parser,
 		g_print ("Parsing videosite for URI '%s' returned '%s'\n", uri, out);
 
 	if (out != NULL) {
-		if (g_str_equal (out, "TOTEM_PL_PARSER_RESULT_ERROR"))
-			return TOTEM_PL_PARSER_RESULT_ERROR;
-		if (g_str_equal (out, "TOTEM_PL_PARSER_RESULT_UNHANDLED"))
-			return TOTEM_PL_PARSER_RESULT_UNHANDLED;
+		if (g_str_equal (out, "TOTEM_PL_PARSER_RESULT_ERROR")) {
+			ret = TOTEM_PL_PARSER_RESULT_ERROR;
+			goto out;
+		}
+		if (g_str_equal (out, "TOTEM_PL_PARSER_RESULT_UNHANDLED")) {
+			ret = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+			goto out;
+		}
 	} else {
 		/* totem-pl-parser-videosite failed to launch */
-		return TOTEM_PL_PARSER_RESULT_ERROR;
+		ret = TOTEM_PL_PARSER_RESULT_ERROR;
+		goto out;
 	}
 
 	ht = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -133,7 +186,12 @@ totem_pl_parser_add_videosite (TotemPlParser *parser,
 	totem_pl_parser_add_hash_table (parser, ht, new_uri, FALSE);
 	g_free (new_uri);
 
-	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	ret = TOTEM_PL_PARSER_RESULT_SUCCESS;
+
+out:
+	g_free (script);
+	g_free (uri);
+	return ret;
 #else
 	return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 #endif /* !HAVE_QUVI */
