@@ -196,6 +196,7 @@ static PlaylistTypes special_types[] = {
 	PLAYLIST_TYPE ("audio/x-scpls", totem_pl_parser_add_pls, NULL, FALSE),
 	PLAYLIST_TYPE ("application/x-smil", totem_pl_parser_add_smil, NULL, FALSE),
 	PLAYLIST_TYPE ("application/smil", totem_pl_parser_add_smil, NULL, FALSE),
+	PLAYLIST_TYPE ("application/smil+xml", totem_pl_parser_add_smil, NULL, FALSE),
 	PLAYLIST_TYPE ("application/vnd.ms-wpl", totem_pl_parser_add_smil, NULL, FALSE),
 	PLAYLIST_TYPE ("video/x-ms-wvx", totem_pl_parser_add_asx, NULL, FALSE),
 	PLAYLIST_TYPE ("audio/x-ms-wax", totem_pl_parser_add_asx, NULL, FALSE),
@@ -571,6 +572,10 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "Whether the track is playing", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("audio-track", "audio-track",
+				     "The default audio-track to play", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -1557,8 +1562,10 @@ totem_pl_parser_scheme_is_ignored (TotemPlParser *parser, GFile *uri)
 	g_mutex_lock (&parser->priv->ignore_mutex);
 
 	scheme = g_file_get_uri_scheme (uri);
-	if (!scheme)
+	if (!scheme) {
+		g_mutex_unlock (&parser->priv->ignore_mutex);
 		return TRUE;
+	}
 	ret = GPOINTER_TO_INT (g_hash_table_lookup (parser->priv->ignore_schemes, scheme));
 	g_free (scheme);
 
@@ -1731,10 +1738,16 @@ totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetyp
 		return FALSE;
 
 	for (i = 0; i < G_N_ELEMENTS (ignore_types); i++) {
-		if (g_content_type_is_a (mimetype, ignore_types[i].mimetype) != FALSE)
+		if (g_content_type_is_a (mimetype, ignore_types[i].mimetype) != FALSE) {
+			if (parser->priv->debug)
+				g_print ("Ignoring %s because it's a %s\n", mimetype, ignore_types[i].mimetype);
 			return TRUE;
-		if (g_content_type_equals (mimetype, ignore_types[i].mimetype) != FALSE)
+		}
+		if (g_content_type_equals (mimetype, ignore_types[i].mimetype) != FALSE) {
+			if (parser->priv->debug)
+				g_print ("Ignoring %s because it's equal to %s\n", mimetype, ignore_types[i].mimetype);
 			return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -1877,6 +1890,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 		g_free (data);
 		g_free (mimetype);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	} else if (strcmp (mimetype, HLS_MIME_TYPE) == 0) {
+		g_free (data);
+		g_free (mimetype);
+		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
 	/* If we're at the top-level of the parsing, try to get more
